@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003-2007, 2009-2012 by                            */
+/*  Copyright 1996-2000, 2003-2007, 2009-2013 by                            */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -23,7 +23,6 @@
 #include FT_SYNTHESIS_H
 
 #define MAXPTSIZE      500                 /* dtp */
-#define HEADER_HEIGHT  8
 
 #undef  CEIL
 #define CEIL( x )  ( ( (x) + 63 ) >> 6 )
@@ -71,8 +70,11 @@
   }
 #endif
 
-  typedef struct  status_
+  typedef struct  GridStatusRec_
   {
+    int          width;
+    int          height;
+
     int          ptsize;
     int          res;
     int          Num;  /* glyph index */
@@ -114,24 +116,17 @@
 
   static GridStatusRec  status;
 
+
   static void
-  grid_status_init( GridStatus       st,
-                    FTDemo_Display*  display )
+  grid_status_init( GridStatus  st )
   {
+    st->width         = DIM_X;
+    st->height        = DIM_Y;
+
     st->scale         = 1.0;
-    st->x_origin      = display->bitmap->width / 4;
-    st->y_origin      = display->bitmap->rows / 4;
+    st->x_origin      = 0;
+    st->y_origin      = 0;
     st->margin        = 0.05;
-    st->axis_color    = grFindColor( display->bitmap,   0,   0,   0, 255 );
-    st->grid_color    = grFindColor( display->bitmap, 192, 192, 192, 255 );
-    st->outline_color = grFindColor( display->bitmap, 255,   0,   0, 255 );
-    st->on_color      = grFindColor( display->bitmap,  64,  64, 255, 255 );
-    st->conic_color   = grFindColor( display->bitmap,   0, 128,   0, 255 );
-    st->cubic_color   = grFindColor( display->bitmap, 255,  64, 255, 255 );
-    st->segment_color = grFindColor( display->bitmap,  64, 255, 128,  64 );
-    st->disp_width    = display->bitmap->width;
-    st->disp_height   = display->bitmap->rows;
-    st->disp_bitmap   = display->bitmap;
 
     st->do_horz_hints = 1;
     st->do_vert_hints = 1;
@@ -140,9 +135,27 @@
     st->do_outline    = 1;
     st->do_segment    = 0;
 
-    st->Num    = 0;
-    st->gamma  = 1.0;
-    st->header = "";
+    st->Num           = 0;
+    st->gamma         = 1.0;
+    st->header        = "";
+  }
+
+
+  static void
+  grid_status_display( GridStatus       st,
+                       FTDemo_Display*  display )
+  {
+    st->disp_width    = display->bitmap->width;
+    st->disp_height   = display->bitmap->rows;
+    st->disp_bitmap   = display->bitmap;
+
+    st->axis_color    = grFindColor( display->bitmap,   0,   0,   0, 255 );
+    st->grid_color    = grFindColor( display->bitmap, 192, 192, 192, 255 );
+    st->outline_color = grFindColor( display->bitmap, 255,   0,   0, 255 );
+    st->on_color      = grFindColor( display->bitmap,  64,  64, 255, 255 );
+    st->conic_color   = grFindColor( display->bitmap,   0, 128,   0, 255 );
+    st->cubic_color   = grFindColor( display->bitmap, 255,  64, 255, 255 );
+    st->segment_color = grFindColor( display->bitmap,  64, 255, 128,  64 );
   }
 
 
@@ -511,9 +524,10 @@
         FT_Get_Glyph( slot, &glyph );
         FT_Glyph_Stroke( &glyph, stroker, 1 );
 
-        FTDemo_Draw_Glyph_Color( handle, display, glyph, &ox, &oy,
-                                 st->outline_color );
-        FT_Done_Glyph( glyph );
+        error = FTDemo_Draw_Glyph_Color( handle, display, glyph, &ox, &oy,
+                                         st->outline_color );
+        if ( !error )
+          FT_Done_Glyph( glyph );
       }
 
       /* now draw the points */
@@ -569,6 +583,7 @@
 
 
     FTDemo_Display_Clear( display );
+    grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( display->bitmap );
@@ -998,7 +1013,8 @@
       "\n" );
     fprintf( stderr,
       "Usage: %s [options] pt font ...\n"
-      "\n", execname );
+      "\n",
+             execname );
     fprintf( stderr,
       "  pt        The point size for the given resolution.\n"
       "            If resolution is 72dpi, this directly gives the\n"
@@ -1009,9 +1025,16 @@
       "            the corresponding metrics file (with extension\n"
       "            `.afm' or `.pfm').\n"
       "\n" );
+    fprintf( stderr,
+      "  -w W      Set the window width to W pixels (default: %dpx).\n"
+      "  -h H      Set the window height to H pixels (default: %dpx).\n"
+      "\n",
+             DIM_X, DIM_Y );
     fprintf( stderr,
       "  -r R      Use resolution R dpi (default: 72dpi).\n"
       "  -f index  Specify first index to display (default: 0).\n"
+      "\n"
+      "  -v        Show version."
       "\n" );
 
     exit( 1 );
@@ -1030,7 +1053,7 @@
 
     while ( 1 )
     {
-      option = getopt( *argc, *argv, "f:r:" );
+      option = getopt( *argc, *argv, "f:h:r:vw:" );
 
       if ( option == -1 )
         break;
@@ -1041,12 +1064,39 @@
         status.Num = atoi( optarg );
         break;
 
+      case 'h':
+        status.height = atoi( optarg );
+        if ( status.height < 1 )
+          usage( execname );
+        break;
+
       case 'r':
         status.res = atoi( optarg );
         if ( status.res < 1 )
           usage( execname );
         break;
 
+      case 'v':
+        {
+          FT_Int  major, minor, patch;
+
+
+          FT_Library_Version( handle->library, &major, &minor, &patch );
+
+          printf( "ftgrid (FreeType) %d.%d", major, minor );
+          if ( patch )
+            printf( ".%d", patch );
+          printf( "\n" );
+          exit( 0 );
+        }
+        break;
+
+      case 'w':
+        status.width = atoi( optarg );
+        if ( status.width < 1 )
+          usage( execname );
+        break;
+
       default:
         usage( execname );
         break;
@@ -1078,22 +1128,22 @@
     grEvent  event;
 
 
-    display = FTDemo_Display_New( gr_pixel_mode_rgb24 );
+    /* initialize engine */
+    handle = FTDemo_New();
+
+    grid_status_init( &status );
+    parse_cmdline( &argc, &argv );
+
+    display = FTDemo_Display_New( gr_pixel_mode_rgb24,
+                                  status.width, status.height );
     if ( !display )
       Fatal( "could not allocate display surface" );
 
-    memset( display->fore_color.chroma, 0, 4 );
-    memset( display->back_color.chroma, 0xff, 4 );
+    grid_status_display( &status, display );
+
     grSetTitle( display->surface,
                 "FreeType Glyph Grid Viewer - press F1 for help" );
 
-    grid_status_init( &status, display );
-
-    parse_cmdline( &argc, &argv );
-
-    /* initialize engine */
-    handle = FTDemo_New( FT_ENCODING_NONE );
-
     for ( ; argc > 0; argc--, argv++ )
       FTDemo_Install_Font( handle, argv[0] );
 
